"""
This example shows how to embed a Chaco QWidget inside the jigna view using
an <object> tag.
"""

#### Imports ####

from chaco.api import Plot, ArrayPlotData
from jigna.api import HTMLWidget, Template
from jigna.qt import QtGui
from numpy import linspace, sin
from traits.api import (
    HasTraits, CInt, Instance, Array, Property, on_trait_change
)

#### Domain model ####

class DomainModel(HasTraits):
    """
    The algorithmic domain model which specifies the mathematical relationship
    between x and y.
    """

    #: Independent variable of the domain equation
    x = Array
    def _x_default(self):
        return linspace(-14, 14, 1000)

    #: Dependent variable of the domain equation
    y = Property(Array, depends_on=['x', 'scaling_factor'])
    def _get_y(self):
        return sin(self.scaling_factor * self.x) * self.x**3

    #: A scaling factor to tune the output
    scaling_factor = CInt

#### Controller layer ####

class PlotController(HasTraits):
    """
    A Controller class which creates a Chaco plot object for the given domain
    model.
    """

    #: Instance of the domain model which is being displayed by this controller
    domain_model = Instance(DomainModel)

    # The Chaco Plot object. This is the object which is usually visualized via
    # traitsui using the enable ComponentEditor
    plot  = Instance(Plot)
    def _plot_default(self):
        plot = Plot(ArrayPlotData(x=self.domain_model.x, y=self.domain_model.y))
        plot.plot(("x", "y"), type="line", color="blue")

        return plot

    @on_trait_change('domain_model.scaling_factor')
    def update_plot(self):
        self.plot.data.set_data('x', self.domain_model.x)
        self.plot.data.set_data('y', self.domain_model.y)

    def create_plot_widget(self):
        """ This method is used as a factory to create the QWidget for the Chaco
        'plot' component.

        This is the standard way to obtain the QWidget out of a Chaco Plot object.
        We create a hidden traitsui view and return it's 'control' to obtain the
        required QWidget.
        """
        from traitsui.api import View, Item
        from enable.api import ComponentEditor

        view = View(Item('plot', editor=ComponentEditor(), show_label=False))
        ui = self.edit_traits(view=view, parent=None, kind='subpanel')

        return ui.control

#### UI layer ####

body_html = """
    <div>
      Scaling factor: <input type="range" ng-model="domain_model.scaling_factor"
                       min=0 max=100><br>
      Plot:<br>

      <!-- Specify type='application/x-qwidget' to embed a QWidget in the jigna
      view. Note that 'widget-factory' here refers to the factory method in
      Python which will generate that QWidget. -->
      <div>
        <object type="application/x-qwidget"
                widget-factory="plot_controller.create_plot_widget"
                width="500" height="400">
        </object>
      </div>
    </div>
"""

template = Template(body_html=body_html, recommended_size=(600, 600))

#### Entry point ####

def main():
    # Start the Qt application
    app = QtGui.QApplication.instance() or QtGui.QApplication([])

    # Instantiate the domain model and the plot controller
    domain_model = DomainModel(scaling_factor=50)
    plot_controller = PlotController(domain_model=domain_model)

    # Create the jigna based HTML widget which renders the given HTML template
    # with the given context.
    #
    # The widget contains an embedded Chaco QWidget showing a 2D plot of
    # the domain model. Moving the slider on the UI changes the domain model
    # and hence the Chaco plot.
    widget = HTMLWidget(
        template=template,
        context={
            'domain_model': domain_model,
            'plot_controller': plot_controller
        }
    )
    widget.show()

    # Start the event loop
    app.exec_()

if __name__ == "__main__":
    main()

#### EOF ######################################################################



"""
This example shows how to serve a simple jigna view over the web.
"""

#### Imports ####
from __future__ import print_function

from tornado.ioloop import IOLoop
from traits.api import HasTraits, Int, Str
from jigna.web_app import WebApp
from jigna.template import Template

#### Domain model ####

class Person(HasTraits):
    name = Str
    age  = Int

#### UI layer ####

body_html = """
    <div>
      Name: <input ng-model="person.name"><br>
      Age: <input ng-model="person.age" type='number'>
    </div>
"""

template = Template(body_html=body_html)

#### Entry point ####

def main():
    # Start the tornado ioloop application
    ioloop = IOLoop.instance()

    # Instantiate the domain model
    fred = Person(name='Fred', age=42)

    # Create a web app serving the view with the domain model added to its
    # context.
    app = WebApp(template=template, context={'person': fred})
    app.listen(8000)

    # Start serving the web app on port 8000.
    #
    # Point your web browser to http://localhost:8000/ to connect to this jigna
    # web app. Any operation performed on the client directly update the
    # model attributes on the server.
    print('Serving on port 8000...')
    ioloop.start()

if __name__ == "__main__":
    main()

#### EOF ######################################################################



#
# Jigna product code
#
# (C) Copyright 2013 Enthought, Inc., Austin, TX
# All right reserved.
#

# Standard library imports.
import inspect
import logging

# System library imports.
from jigna.qt import QtCore

# Logger.
logger = logging.getLogger(__name__)

def create_js_object_wrapper(callbacks=[], parent=None):
    """ Create a JS object wrapper containing the given callbacks as its
    methods.

    Note: Set the parent (setParent()) of the returned QObject to make sure
    it is destroyed when the parent is destroyed, or manually destroy the
    object when it is no longer needed (such as when the `window` js object
    is cleared or the webview/webpage/webframe object is destroyed);
    else stray references to the object may cause memory leaks.

    """
    # Create the container class dict.
    class_dict = {}

    # Create the container callback slots, these need to be defined on the
    # class (associated with the QMetaObject).
    slot_map = {}
    for _callback in callbacks:
        name, callback = _callback

        wrapped = wrap_func(callback, name)
        if wrapped:
            class_dict[name] = wrapped
            slot_map[wrapped.name] = callback
        else:
            logger.error('Callback %r is not translatable to JavaScript', name)

    # Create the container class.
    container_class = type(
        'CustomPythonContainer', (_PythonContainer, QtCore.QObject,), class_dict
    )
    qobj = container_class(parent=parent, slot_map=slot_map)

    return qobj

def wrap_func(func, name=None):
    """ Wraps a Python callable as a Qt Slot.
    """
    try:
        args = inspect.getargspec(func).args
    except TypeError:
        return None

    if args and args[0] in ('self', 'cls'):
        args.pop(0)
    types = ['QVariant'] * len(args)

    slot_key = name if name is not None else 'func_' + hex(id(func))

    @QtCore.Slot(*types, name=name, result='QVariant')
    def wrapped(self, *args):
        return self._slot_map[slot_key](*args)
    wrapped.name = slot_key
    return wrapped

#### Private protocol #########################################################

class _PythonContainer(QtCore.QObject):
    """ Container class for python object to be exposed to js. """
    def __init__(self, parent=None, slot_map=None):
        super(_PythonContainer, self).__init__(parent)

        # slot_name: callable
        self._slot_map = slot_map


#
# Enthought product code
#
# (C) Copyright 2013-2016 Enthought, Inc., Austin, TX
# All right reserved.
#

from textwrap import dedent

# Local imports
from .template import Template


class VueTemplate(Template):
    """A template for vue.js templates.  Note that this assumes that the
    ViewModel is attached to the body and all the jigna models are exposed.
    """
    def _html_template_default(self):
        return dedent("""
        <html>
          <head>
            <script type="text/javascript" src="/jigna/jigna-vue.js"></script>
            {head_html}
          </head>

          <body>
            {body_html}

          <script type="text/javascript">
              var vm = undefined;
              // jigna.models are ready only when the deferred returned by initialize
              // is resolved. One could also use jigna.ready.done.
              jigna.initialize({{async: {async}}}).done(function() {{
                  vm = new Vue({{
                      el: 'body',
                      data: jigna.models,
                      methods: {{
                        threaded: function(obj, method_name, args) {{
                           jigna.threaded.apply(jigna, arguments);
                        }},
                      }}
                  }});
              }});
          </script>
          </body>

        </html>
        """)



"""
This example shows how to include HTML templates for creating reusable
components.
"""

#### Imports ####

from traits.api import HasTraits, Int, Str
from jigna.api import HTMLWidget, Template
from jigna.qt import QtGui

#### Domain model ####

class Person(HasTraits):
    name = Str
    age  = Int

#### UI layer ####

html = """
  <html ng-app='templating'>
    <head>
      <script type='text/javascript' src='/jigna/jigna.js'></script>
      <script type='text/javascript' src='ex16_templating.js'></script>
    </head>
    <body>
      <person-view person="fred"></person-view>
      <person-view person="wilma"></person-view>
    </body>
  </html>
"""

# The base_url field specifies where to look when trying to get external
# resources(defaults to an empty string, i.e. the current directory)
template = Template(html=html)

#### Entry point ####


def main():
    # Start the Qt application
    app = QtGui.QApplication([])

    # Instantiate the domain model
    fred = Person(name='Fred', age=28)
    wilma = Person(name='Wilma', age=25)

    # Create the jigna based HTML widget which renders the given HTML template
    # with the given context.
    widget = HTMLWidget(
        template=template, context={'fred': fred, 'wilma': wilma}
    )
    widget.show()

    # Start the event loop.
    #
    # You should see that the person-view component's template is rendered with
    # the correct domain models.
    app.exec_()

if __name__ == "__main__":
    main()

#### EOF ######################################################################
